package main

import (
	"runtime"
	"flag"
	"fmt"
	"os"
	"os/user"
	"strconv"
	"net/textproto"
	"strings"
//	"time"
)

var debug *bool
var command chan string = make(chan string)
var exit chan bool

var sc ServerConfig
var uc UserConfig

type ServerConfig struct {
	Server string
	Port int
	Channel string
}

type UserConfig struct {
	NickName string
	UserName string
	HostName string
	FullName string
}

func run(c *textproto.Conn) {
	var com string
	for {
		select {
		case com  = <- command:
			ComDispatch(c, com)
		default:
			if res, err := c.ReadLine(); err != nil {
				if *debug {
					fmt.Println(err)
				}
				os.Exit(0)
			} else {
				Response(c, res)
			}
		}
	}
}

func Response(c *textproto.Conn, res string) {
	fmt.Println(res)
	switch {
	case strings.HasPrefix(res, "PING"):
		f := strings.Split(res, ":")
		if *debug {
			fmt.Println(f)
		}
		if _, err := c.Cmd("PONG :%s", f[1]); err != nil {
			if *debug {
				fmt.Println(err)
			}
			os.Exit(0)
		}
	case strings.HasPrefix(res, ":"):
		f := strings.Split(res, ":")
		n := strings.Split(f[1], "!")
		switch {
		case strings.Contains(f[1], "PRIVMSG"):
			if *debug {
				fmt.Println(f)
				fmt.Println(n)
			}
			switch {
			case strings.Contains(res, "蛤蟆"):
				if _, err := c.Cmd("PRIVMSG #%s : %s : 代表我主人诅咒你！", sc.Channel, n[0]); err != nil {
					if *debug {
						fmt.Println(err)
					}
					os.Exit(0)
				}
			case strings.Contains(res, "践踏"):
				if strings.Contains(f[len(f)-1],"hamo") {
					c.Cmd("PRIVMSG #%s : 修好了我!", sc.Channel)
					break
				}
				if _, err := c.Cmd("PRIVMSG #%s : %s : 代表主人践踏你！", sc.Channel, f[len(f)-1]); err != nil {
					if *debug {
						fmt.Println(err)
					}
					os.Exit(0)
				}

			case strings.Contains(res, uc.NickName):
				if _, err := c.Cmd("PRIVMSG #%s : hi! %s", sc.Channel, n[0]); err != nil {
					if *debug {
						fmt.Println(err)
					}
					os.Exit(0)
				}
			default:
			}	
		default:
		}
	default:
	}
}

func ComDispatch(conn *textproto.Conn, com string) (err error) {
	switch com {
	case "NICK":
		if _, err := conn.Cmd("NICK %s", uc.NickName); err != nil {
			return err
		}
	case "USER":
		if _, err := conn.Cmd("USER %s 8 %s : %s", uc.UserName, uc.HostName, uc.FullName); err != nil {
			return err
		}
	case "JOIN":
		if _, err := conn.Cmd("JOIN #%s", sc.Channel); err != nil {
			return err
		}
	case "PING":
		
	case "EXIT":
		fmt.Println("EXIT")
		os.Exit(0)
	}
	return
}


func IRCconn(sc ServerConfig) (c *textproto.Conn, err error) {
	url := sc.Server + ":" + strconv.Itoa(sc.Port)
	
	if *debug {
		fmt.Println("URL is :", url)
	}
	
	if c, err  = textproto.Dial("tcp", url); err != nil {
		if *debug {
			fmt.Println("CONNECT FAILED")
			fmt.Println(err)
		}
		return nil, err
	} else {
		if *debug {
			fmt.Println("CONNECT GOT")
		}
		return c, nil
	}
	return
}

func arginit() (server, nick, channel *string, port *int) {
	server = flag.String("server", "chat.freenode.net", "Set the server")
	port = flag.Int("port", 6667, "Set the port")
	nick = flag.String("nick", "gobot_hamo", "Set the bot nickname")
	debug = flag.Bool("debug", false, "debug")
	channel = flag.String("c", "ubuntu-cn", "set the channel")

	flag.Usage = func() {
		fmt.Println("Usage: ", os.Args[0], "--server <Server address> --port <Server Port> --nick <bot name> --debug -c <channel>")
	}
	
	flag.Parse()
	return
}

func confinit() (sc ServerConfig, uc UserConfig) {
	s, n, c, p  := arginit()
	
	sc.Server = *s
	sc.Port = *p
	sc.Channel = *c

	uc.NickName = *n
	
	uid := os.Getuid()
	ui, err := user.LookupId(strconv.Itoa(uid))
	if err != nil {
		fmt.Println(err)
		return
	}
	fmt.Println(ui)
	uc.UserName = ui.Username
	uc.FullName = ui.Name
	uc.HostName, _ = os.Hostname()
	return
}

func main() {
	runtime.GOMAXPROCS(50)
	sc, uc = confinit()


	fmt.Println(sc)
	fmt.Println(uc)
	conn, err := IRCconn(sc)
	defer conn.Close()

	if err != nil {
		fmt.Println(err)
		return
	}

	go run(conn)

	command <- "NICK"

	command <- "USER"

	command <- "JOIN"

	<- exit

}